// SPDX-License-Identifier: GPL-2.1-or-later
/*
 * gp_get_put_pixel.gen.h
 *
 * GENERATED on 2023 10 30 20:24:52 from gp_get_put_pixel.gen.h.t
 *
 * DO NOT MODIFY THIS FILE DIRECTLY!
 */
#ifndef GP_GET_PUT_PIXEL_GEN_H
#define GP_GET_PUT_PIXEL_GEN_H

/*
 * Access pixel bytes, Get and PutPixel
 * Do not include directly, use gp_pixel.h
 *
 * Copyright (C) 2011-2014 Cyril Hrubis <metan@ucw.cz>
 * Copyright (C) 2011      Tomas Gavenciak <gavento@ucw.cz>
 */

 /*

   Note about byte aligment
   ~~~~~~~~~~~~~~~~~~~~~~~~

   Unaligned access happens when instruction that works with multiple byte
   value gets an address that is not divideable by the size of the value. Eg.
   if 32 bit integer instruction gets an address that is not a multiple of 4.
   On intel cpus this type of access works and is supported however the C
   standard defines this as undefined behavior. This fails to work ARM and most
   of the non intel cpus. So some more trickery must be done in order to write
   unaligned multibyte values. First of all we must compute offset and number
   of bytes to be accessed (which is cruicial for speed as we are going to read
   the pixel value byte by byte).

   The offsets (starting with the first one eg. pixel_size mod 8) forms subgroup
   in the mod 8 cyclic group. The maximal count of bits, from the start of the
   byte, then will be max from this subgroup + pixel_size. If this number is
   less or equal to 8 * N, we could write such pixel by writing N bytes.

   For example the offsets of 16 BPP forms subgroup only with {0} so we only
   need 2 bytes to write it. As a matter of fact the 16 and 32 BPP are special
   cases that are always aligned together with the 8 BPP (which is aligned
   trivially). These three are coded as special cases which yields to faster
   operations in case of 16 and 32 BPP. The 24 BPP is not aligned as there are
   no instruction to operate 3 byte long numbers.

   For second example take offsets of 20 BPP that forms subgroup {4, 0}
   so the max + pixel_size = 24 and indeed we fit into 3 bytes.

   If pixel_size is coprime to 8, the offsets generates whole group and so the
   max + pixel_size = 7 + pixel_size. The 17 BPP fits into 24 bits and so 3
   bytes are needed. The 19 BPP fits into 26 bits and because of that 4 bytes
   are needed.

   Once we figure maximal number of bytes and the offset all that is to be done
   is to fetch first and last byte to combine it together with given pixel value
   and write the result back to the bitmap.

 */

#include <core/gp_get_set_bits.h>
#include "core/gp_pixmap.h"

/*
 * macro to get address of pixel in a 1BPP_DB pixmap
 */
#define GP_PIXEL_ADDR_1BPP_DB(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(1 * ((pixmap)->offset + x)) / 8))

/*
 * macro to get bit-shift of pixel in 1BPP_DB pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_1BPP_DB(pixmap, x) \
	(((x + (pixmap)->offset) % 8) * 1)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_1BPP_DB(pixmap, x) \
	(((pixmap)->offset + x) % 8)

/*
 * gp_getpixel for 1BPP_DB
 */
static inline gp_pixel gp_getpixel_raw_1BPP_DB(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * Whole pixel is stored in one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_1BPP_DB(c, x), 1,
		*(GP_PIXEL_ADDR_1BPP_DB(c, x, y)));
}

/*
 * gp_putpixel for 1BPP_DB
 */
static inline void gp_putpixel_raw_1BPP_DB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * Whole pixel is stored one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_1BPP_DB(c, x), 1,
	                     GP_PIXEL_ADDR_1BPP_DB(c, x, y), p);
}

static inline void gp_putpixel_raw_clipped_1BPP_DB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_1BPP_DB(c, x, y, p);
}
/*
 * macro to get address of pixel in a 1BPP_UB pixmap
 */
#define GP_PIXEL_ADDR_1BPP_UB(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(1 * ((pixmap)->offset + x)) / 8))

/*
 * macro to get bit-shift of pixel in 1BPP_UB pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_1BPP_UB(pixmap, x) \
	(7 - ((x + (pixmap)->offset) % 8) * 1)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_1BPP_UB(pixmap, x) \
	(((pixmap)->offset + x) % 8)

/*
 * gp_getpixel for 1BPP_UB
 */
static inline gp_pixel gp_getpixel_raw_1BPP_UB(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * Whole pixel is stored in one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_1BPP_UB(c, x), 1,
		*(GP_PIXEL_ADDR_1BPP_UB(c, x, y)));
}

/*
 * gp_putpixel for 1BPP_UB
 */
static inline void gp_putpixel_raw_1BPP_UB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * Whole pixel is stored one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_1BPP_UB(c, x), 1,
	                     GP_PIXEL_ADDR_1BPP_UB(c, x, y), p);
}

static inline void gp_putpixel_raw_clipped_1BPP_UB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_1BPP_UB(c, x, y, p);
}
/*
 * macro to get address of pixel in a 2BPP_DB pixmap
 */
#define GP_PIXEL_ADDR_2BPP_DB(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(2 * ((pixmap)->offset + x)) / 8))

/*
 * macro to get bit-shift of pixel in 2BPP_DB pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_2BPP_DB(pixmap, x) \
	(((x + (pixmap)->offset) % 4) * 2)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_2BPP_DB(pixmap, x) \
	(((pixmap)->offset + x) % 4)

/*
 * gp_getpixel for 2BPP_DB
 */
static inline gp_pixel gp_getpixel_raw_2BPP_DB(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * Whole pixel is stored in one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_2BPP_DB(c, x), 2,
		*(GP_PIXEL_ADDR_2BPP_DB(c, x, y)));
}

/*
 * gp_putpixel for 2BPP_DB
 */
static inline void gp_putpixel_raw_2BPP_DB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * Whole pixel is stored one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_2BPP_DB(c, x), 2,
	                     GP_PIXEL_ADDR_2BPP_DB(c, x, y), p);
}

static inline void gp_putpixel_raw_clipped_2BPP_DB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_2BPP_DB(c, x, y, p);
}
/*
 * macro to get address of pixel in a 2BPP_UB pixmap
 */
#define GP_PIXEL_ADDR_2BPP_UB(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(2 * ((pixmap)->offset + x)) / 8))

/*
 * macro to get bit-shift of pixel in 2BPP_UB pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_2BPP_UB(pixmap, x) \
	(6 - ((x + (pixmap)->offset) % 4) * 2)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_2BPP_UB(pixmap, x) \
	(((pixmap)->offset + x) % 4)

/*
 * gp_getpixel for 2BPP_UB
 */
static inline gp_pixel gp_getpixel_raw_2BPP_UB(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * Whole pixel is stored in one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_2BPP_UB(c, x), 2,
		*(GP_PIXEL_ADDR_2BPP_UB(c, x, y)));
}

/*
 * gp_putpixel for 2BPP_UB
 */
static inline void gp_putpixel_raw_2BPP_UB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * Whole pixel is stored one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_2BPP_UB(c, x), 2,
	                     GP_PIXEL_ADDR_2BPP_UB(c, x, y), p);
}

static inline void gp_putpixel_raw_clipped_2BPP_UB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_2BPP_UB(c, x, y, p);
}
/*
 * macro to get address of pixel in a 4BPP_DB pixmap
 */
#define GP_PIXEL_ADDR_4BPP_DB(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(4 * ((pixmap)->offset + x)) / 8))

/*
 * macro to get bit-shift of pixel in 4BPP_DB pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_4BPP_DB(pixmap, x) \
	(((x + (pixmap)->offset) % 2) * 4)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_4BPP_DB(pixmap, x) \
	(((pixmap)->offset + x) % 2)

/*
 * gp_getpixel for 4BPP_DB
 */
static inline gp_pixel gp_getpixel_raw_4BPP_DB(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * Whole pixel is stored in one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_4BPP_DB(c, x), 4,
		*(GP_PIXEL_ADDR_4BPP_DB(c, x, y)));
}

/*
 * gp_putpixel for 4BPP_DB
 */
static inline void gp_putpixel_raw_4BPP_DB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * Whole pixel is stored one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_4BPP_DB(c, x), 4,
	                     GP_PIXEL_ADDR_4BPP_DB(c, x, y), p);
}

static inline void gp_putpixel_raw_clipped_4BPP_DB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_4BPP_DB(c, x, y, p);
}
/*
 * macro to get address of pixel in a 4BPP_UB pixmap
 */
#define GP_PIXEL_ADDR_4BPP_UB(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(4 * ((pixmap)->offset + x)) / 8))

/*
 * macro to get bit-shift of pixel in 4BPP_UB pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_4BPP_UB(pixmap, x) \
	(4 - ((x + (pixmap)->offset) % 2) * 4)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_4BPP_UB(pixmap, x) \
	(((pixmap)->offset + x) % 2)

/*
 * gp_getpixel for 4BPP_UB
 */
static inline gp_pixel gp_getpixel_raw_4BPP_UB(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * Whole pixel is stored in one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	return GP_GET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_4BPP_UB(c, x), 4,
		*(GP_PIXEL_ADDR_4BPP_UB(c, x, y)));
}

/*
 * gp_putpixel for 4BPP_UB
 */
static inline void gp_putpixel_raw_4BPP_UB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * Whole pixel is stored one byte
	 *
	 * The full list = {1, 2, 4}
	 */
	GP_SET_BITS1_ALIGNED(GP_PIXEL_ADDR_SHIFT_4BPP_UB(c, x), 4,
	                     GP_PIXEL_ADDR_4BPP_UB(c, x, y), p);
}

static inline void gp_putpixel_raw_clipped_4BPP_UB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_4BPP_UB(c, x, y, p);
}
/*
 * macro to get address of pixel in a 8BPP pixmap
 */
#define GP_PIXEL_ADDR_8BPP(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(8 * (x)) / 8))

/*
 * macro to get bit-shift of pixel in 8BPP pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_8BPP(pixmap, x) \
	(0)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_8BPP(pixmap, x) \
	(0)

/*
 * gp_getpixel for 8BPP
 */
static inline gp_pixel gp_getpixel_raw_8BPP(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * 8 BPP is byte aligned
	 */
	return *((uint8_t*)GP_PIXEL_ADDR_8BPP(c, x, y));
}

/*
 * gp_putpixel for 8BPP
 */
static inline void gp_putpixel_raw_8BPP(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * 8 BPP is byte aligned
	 */
	*((uint8_t*)GP_PIXEL_ADDR_8BPP(c, x, y)) = p;
}

static inline void gp_putpixel_raw_clipped_8BPP(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_8BPP(c, x, y, p);
}
/*
 * macro to get address of pixel in a 16BPP pixmap
 */
#define GP_PIXEL_ADDR_16BPP(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(16 * (x)) / 8))

/*
 * macro to get bit-shift of pixel in 16BPP pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_16BPP(pixmap, x) \
	(0)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_16BPP(pixmap, x) \
	(0)

/*
 * gp_getpixel for 16BPP
 */
static inline gp_pixel gp_getpixel_raw_16BPP(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * 16 BPP is expected to have aligned pixels
	 */
	return *((uint16_t*)GP_PIXEL_ADDR_16BPP(c, x, y));
}

/*
 * gp_putpixel for 16BPP
 */
static inline void gp_putpixel_raw_16BPP(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * 16 BPP is expected to have aligned pixels
	 */
	*((uint16_t*)GP_PIXEL_ADDR_16BPP(c, x, y)) = p;
}

static inline void gp_putpixel_raw_clipped_16BPP(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_16BPP(c, x, y, p);
}
/*
 * macro to get address of pixel in a 24BPP pixmap
 */
#define GP_PIXEL_ADDR_24BPP(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(24 * (x)) / 8))

/*
 * macro to get bit-shift of pixel in 24BPP pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_24BPP(pixmap, x) \
	(0)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_24BPP(pixmap, x) \
	(0)

/*
 * gp_getpixel for 24BPP
 */
static inline gp_pixel gp_getpixel_raw_24BPP(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * The pixel is stored in two or three bytes
	 *
	 * The max from subgroup (of mod 8 factor group) generated by
	 * pixel_size mod 8 + pixel_size <= 24
	 *
	 * The full list = {11, 13, 14, 15, 17, 18, 20, 24}
	 *
	 * Hint: If the pixel size is coprime to 8 the group is generated by
	 *       pixel_size mod 8 and maximal size thus is pixel_size + 7
	 */
	return GP_GET_BITS3_ALIGNED(GP_PIXEL_ADDR_SHIFT_24BPP(c, x), 24,
		*(GP_PIXEL_ADDR_24BPP(c, x, y)));
}

/*
 * gp_putpixel for 24BPP
 */
static inline void gp_putpixel_raw_24BPP(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * The pixel is stored in two or three bytes
	 *
	 * The max from subgroup (of mod 8 factor group) generated by
	 * pixel_size mod 8 + pixel_size <= 24
	 *
	 * The full list = {11, 13, 14, 15, 17, 18, 20, 24}
	 *
	 * Hint: If the pixel size is coprime to 8 the group is generated by
	 *       pixel_size mod 8 and maximal size thus is pixel_size + 7
	 */
	GP_SET_BITS3_ALIGNED(GP_PIXEL_ADDR_SHIFT_24BPP(c, x), 24,
	                     GP_PIXEL_ADDR_24BPP(c, x, y), p);
}

static inline void gp_putpixel_raw_clipped_24BPP(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_24BPP(c, x, y, p);
}
/*
 * macro to get address of pixel in a 32BPP pixmap
 */
#define GP_PIXEL_ADDR_32BPP(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(32 * (x)) / 8))

/*
 * macro to get bit-shift of pixel in 32BPP pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_32BPP(pixmap, x) \
	(0)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_32BPP(pixmap, x) \
	(0)

/*
 * gp_getpixel for 32BPP
 */
static inline gp_pixel gp_getpixel_raw_32BPP(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * 32 BPP is expected to have aligned pixels
	 */
	return *((uint32_t*)GP_PIXEL_ADDR_32BPP(c, x, y));
}

/*
 * gp_putpixel for 32BPP
 */
static inline void gp_putpixel_raw_32BPP(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * 32 BPP is expected to have aligned pixels
	 */
	*((uint32_t*)GP_PIXEL_ADDR_32BPP(c, x, y)) = p;
}

static inline void gp_putpixel_raw_clipped_32BPP(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_32BPP(c, x, y, p);
}
/*
 * macro to get address of pixel in a 18BPP_DB pixmap
 */
#define GP_PIXEL_ADDR_18BPP_DB(pixmap, x, y) \
	((gp_pixel*)(((void*)((pixmap)->pixels)) + (pixmap)->bytes_per_row * (y) + \
	(18 * ((pixmap)->offset + x)) / 8))

/*
 * macro to get bit-shift of pixel in 18BPP_DB pixmap
 */
#define GP_PIXEL_ADDR_SHIFT_18BPP_DB(pixmap, x) \
	((18 * (x + (pixmap)->offset)) % 8)

/*
 * macro to get pixel offset in pixels
 */
#define GP_PIXEL_ADDR_OFFSET_18BPP_DB(pixmap, x) \
	(0)

/*
 * gp_getpixel for 18BPP_DB
 */
static inline gp_pixel gp_getpixel_raw_18BPP_DB(const gp_pixmap *c, gp_coord x, gp_coord y)
{
	/*
	 * The pixel is stored in two or three bytes
	 *
	 * The max from subgroup (of mod 8 factor group) generated by
	 * pixel_size mod 8 + pixel_size <= 24
	 *
	 * The full list = {11, 13, 14, 15, 17, 18, 20, 24}
	 *
	 * Hint: If the pixel size is coprime to 8 the group is generated by
	 *       pixel_size mod 8 and maximal size thus is pixel_size + 7
	 */
	return GP_GET_BITS3_ALIGNED(GP_PIXEL_ADDR_SHIFT_18BPP_DB(c, x), 18,
		*(GP_PIXEL_ADDR_18BPP_DB(c, x, y)));
}

/*
 * gp_putpixel for 18BPP_DB
 */
static inline void gp_putpixel_raw_18BPP_DB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	/*
	 * The pixel is stored in two or three bytes
	 *
	 * The max from subgroup (of mod 8 factor group) generated by
	 * pixel_size mod 8 + pixel_size <= 24
	 *
	 * The full list = {11, 13, 14, 15, 17, 18, 20, 24}
	 *
	 * Hint: If the pixel size is coprime to 8 the group is generated by
	 *       pixel_size mod 8 and maximal size thus is pixel_size + 7
	 */
	GP_SET_BITS3_ALIGNED(GP_PIXEL_ADDR_SHIFT_18BPP_DB(c, x), 18,
	                     GP_PIXEL_ADDR_18BPP_DB(c, x, y), p);
}

static inline void gp_putpixel_raw_clipped_18BPP_DB(gp_pixmap *c, gp_coord x, gp_coord y, gp_pixel p)
{
	if (GP_PIXEL_IS_CLIPPED(c, x, y))
		return;

	gp_putpixel_raw_18BPP_DB(c, x, y, p);
}
#endif /* GP_GET_PUT_PIXEL_GEN_H */
