// SPDX-License-Identifier: GPL-2.1-or-later
/*
 * gp_mix_pixels2.gen.h
 *
 * GENERATED on 2023 10 30 20:24:52 from gp_mix_pixels2.gen.h.t
 *
 * DO NOT MODIFY THIS FILE DIRECTLY!
 */
#ifndef GP_MIX_PIXELS2_GEN_H
#define GP_MIX_PIXELS2_GEN_H

/*
 * Macros to mix two pixels. The source must have alpha channel.
 *
 * Copyright (C) 2009-2014 Cyril Hrubis <metan@ucw.cz>
 */

#include <core/gp_gamma_correction.h>
#include <core/gp_pixel.h>

//TODO: Fix blit where both source and destination have alpha channel


static inline gp_pixel gp_mix_pixels_RGBA8888_xRGB8888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_xRGB8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_xRGB8888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_RGBA8888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGBA8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGBA8888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_RGB888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_BGR888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_BGR888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_BGR888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_RGB555(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB555_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB555(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_RGB565(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB565_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB565(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_RGB666(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB666_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB666(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_RGB332(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB332_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB332(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_CMYK8888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_CMYK8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_CMYK8888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_G1_DB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_G1_DB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G1_DB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_G2_DB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_G2_DB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G2_DB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_G4_DB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_G4_DB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G4_DB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_G1_UB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_G1_UB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G1_UB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_G2_UB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_G2_UB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G2_UB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_G4_UB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_G4_UB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G4_UB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_G8(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_G8_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G8(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_GA88(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_GA88_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_GA88(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_RGBA8888_G16(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_RGBA8888(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_RGBA8888_TO_RGB888(src, src_rgb);
	GP_PIXEL_G16_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G16(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_xRGB8888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_xRGB8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_xRGB8888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_RGBA8888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGBA8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGBA8888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_RGB888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_BGR888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_BGR888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_BGR888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_RGB555(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB555_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB555(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_RGB565(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB565_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB565(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_RGB666(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB666_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB666(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_RGB332(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_RGB332_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_RGB332(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_CMYK8888(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_CMYK8888_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_CMYK8888(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_G1_DB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_G1_DB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G1_DB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_G2_DB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_G2_DB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G2_DB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_G4_DB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_G4_DB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G4_DB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_G1_UB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_G1_UB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G1_UB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_G2_UB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_G2_UB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G2_UB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_G4_UB(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_G4_UB_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G4_UB(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_G8(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_G8_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G8(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_GA88(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_GA88_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_GA88(dst_rgb, res);

	return res;
}


static inline gp_pixel gp_mix_pixels_GA88_G16(gp_pixel src, gp_pixel dst)
{
	/* Extract the alpha channel */
	unsigned int alpha = GP_PIXEL_GET_A_GA88(src);

	/* Convert the pixel to RGB888, mix the values */
	gp_pixel src_rgb = 0, dst_rgb = 0, res = 0;

	GP_PIXEL_GA88_TO_RGB888(src, src_rgb);
	GP_PIXEL_G16_TO_RGB888(dst, dst_rgb);

	int sr, sg, sb;
	int dr, dg, db;

	sr = GP_PIXEL_GET_R_RGB888(src_rgb);
	sg = GP_PIXEL_GET_G_RGB888(src_rgb);
	sb = GP_PIXEL_GET_B_RGB888(src_rgb);

	dr = GP_PIXEL_GET_R_RGB888(dst_rgb);
	dg = GP_PIXEL_GET_G_RGB888(dst_rgb);
	db = GP_PIXEL_GET_B_RGB888(dst_rgb);


	dr = (dr * (255 - alpha) + sr * alpha + 127) / 255;
	dg = (dg * (255 - alpha) + sg * alpha + 127) / 255;
	db = (db * (255 - alpha) + sb * alpha + 127) / 255;

	dst_rgb = GP_PIXEL_CREATE_RGB888(dr, dg, db);

	GP_PIXEL_RGB888_TO_G16(dst_rgb, res);

	return res;
}

#endif /* GP_MIX_PIXELS2_GEN_H */
