// SPDX-License-Identifier: GPL-2.1-or-later
/*
 * gp_pixel.gen.h
 *
 * GENERATED on 2023 10 30 20:24:52 from gp_pixel.gen.h.t
 *
 * DO NOT MODIFY THIS FILE DIRECTLY!
 */
#ifndef GP_PIXEL_GEN_H
#define GP_PIXEL_GEN_H

/*
 * Pixel type definitions and functions.
 * Do not include directly, use gp_pixel.h
 *
 * Copyright (C) 2011      Tomas Gavenciak <gavento@ucw.cz>
 * Copyright (C) 2013-2023 Cyril Hrubis <metan@ucw.cz>
 */

#include <core/gp_clamp.h>

/*
 * List of all known pixel types
 */
typedef enum gp_pixel_type {
	GP_PIXEL_UNKNOWN,
	GP_PIXEL_xRGB8888,
	GP_PIXEL_RGBA8888,
	GP_PIXEL_RGB888,
	GP_PIXEL_BGR888,
	GP_PIXEL_RGB555,
	GP_PIXEL_RGB565,
	GP_PIXEL_RGB666,
	GP_PIXEL_RGB332,
	GP_PIXEL_CMYK8888,
	GP_PIXEL_P2,
	GP_PIXEL_P4,
	GP_PIXEL_P8,
	GP_PIXEL_G1_DB,
	GP_PIXEL_G2_DB,
	GP_PIXEL_G4_DB,
	GP_PIXEL_G1_UB,
	GP_PIXEL_G2_UB,
	GP_PIXEL_G4_UB,
	GP_PIXEL_G8,
	GP_PIXEL_GA88,
	GP_PIXEL_G16,
	GP_PIXEL_MAX,
} gp_pixel_type;

#define GP_PIXEL_UNKNOWN GP_PIXEL_UNKNOWN
#define GP_PIXEL_xRGB8888 GP_PIXEL_xRGB8888
#define GP_PIXEL_RGBA8888 GP_PIXEL_RGBA8888
#define GP_PIXEL_RGB888 GP_PIXEL_RGB888
#define GP_PIXEL_BGR888 GP_PIXEL_BGR888
#define GP_PIXEL_RGB555 GP_PIXEL_RGB555
#define GP_PIXEL_RGB565 GP_PIXEL_RGB565
#define GP_PIXEL_RGB666 GP_PIXEL_RGB666
#define GP_PIXEL_RGB332 GP_PIXEL_RGB332
#define GP_PIXEL_CMYK8888 GP_PIXEL_CMYK8888
#define GP_PIXEL_P2 GP_PIXEL_P2
#define GP_PIXEL_P4 GP_PIXEL_P4
#define GP_PIXEL_P8 GP_PIXEL_P8
#define GP_PIXEL_G1_DB GP_PIXEL_G1_DB
#define GP_PIXEL_G2_DB GP_PIXEL_G2_DB
#define GP_PIXEL_G4_DB GP_PIXEL_G4_DB
#define GP_PIXEL_G1_UB GP_PIXEL_G1_UB
#define GP_PIXEL_G2_UB GP_PIXEL_G2_UB
#define GP_PIXEL_G4_UB GP_PIXEL_G4_UB
#define GP_PIXEL_G8 GP_PIXEL_G8
#define GP_PIXEL_GA88 GP_PIXEL_GA88
#define GP_PIXEL_G16 GP_PIXEL_G16

/* Automatically generated code for pixel type xRGB8888
 *
 * Size (bpp): 32 (32BPP)
 * Pixel structure: BBBBBBBBGGGGGGGGRRRRRRRRxxxxxxxx
 * Channels:
 *   R  offset:16 size:8
 *   G  offset:8 size:8
 *   B  offset:0 size:8
 */

/*
 * macros to get channels of pixel type xRGB8888
 */
#define GP_PIXEL_GET_R_xRGB8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_G_xRGB8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_B_xRGB8888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to get address and bit-offset of a pixel xRGB8888 in a pixmap
 */
#define GP_PIXEL_ADDR_xRGB8888(pixmap, x, y) GP_PIXEL_ADDR_32BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_xRGB8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros to create gp_pixel of pixel type xRGB8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_xRGB8888(R, G, B) (0\
	+ ((R) << 16) \
	+ ((G) << 8) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type RGBA8888
 *
 * Size (bpp): 32 (32BPP)
 * Pixel structure: AAAAAAAABBBBBBBBGGGGGGGGRRRRRRRR
 * Channels:
 *   R  offset:24 size:8
 *   G  offset:16 size:8
 *   B  offset:8 size:8
 *   A  offset:0 size:8
 */

/*
 * macros to get channels of pixel type RGBA8888
 */
#define GP_PIXEL_GET_R_RGBA8888(p) (GP_GET_BITS(24, 8, (p)))
#define GP_PIXEL_GET_G_RGBA8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_B_RGBA8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_A_RGBA8888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to get address and bit-offset of a pixel RGBA8888 in a pixmap
 */
#define GP_PIXEL_ADDR_RGBA8888(pixmap, x, y) GP_PIXEL_ADDR_32BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGBA8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros to create gp_pixel of pixel type RGBA8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGBA8888(R, G, B, A) (0\
	+ ((R) << 24) \
	+ ((G) << 16) \
	+ ((B) << 8) \
	+ ((A) << 0) \
	)

/* Automatically generated code for pixel type RGB888
 *
 * Size (bpp): 24 (24BPP)
 * Pixel structure: BBBBBBBBGGGGGGGGRRRRRRRR
 * Channels:
 *   R  offset:16 size:8
 *   G  offset:8 size:8
 *   B  offset:0 size:8
 */

/*
 * macros to get channels of pixel type RGB888
 */
#define GP_PIXEL_GET_R_RGB888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_G_RGB888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_B_RGB888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to get address and bit-offset of a pixel RGB888 in a pixmap
 */
#define GP_PIXEL_ADDR_RGB888(pixmap, x, y) GP_PIXEL_ADDR_24BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB888(x) GP_PIXEL_ADDR_OFFSET_24BPP(x)

/*
 * macros to create gp_pixel of pixel type RGB888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB888(R, G, B) (0\
	+ ((R) << 16) \
	+ ((G) << 8) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type BGR888
 *
 * Size (bpp): 24 (24BPP)
 * Pixel structure: RRRRRRRRGGGGGGGGBBBBBBBB
 * Channels:
 *   B  offset:16 size:8
 *   G  offset:8 size:8
 *   R  offset:0 size:8
 */

/*
 * macros to get channels of pixel type BGR888
 */
#define GP_PIXEL_GET_B_BGR888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_G_BGR888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_R_BGR888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to get address and bit-offset of a pixel BGR888 in a pixmap
 */
#define GP_PIXEL_ADDR_BGR888(pixmap, x, y) GP_PIXEL_ADDR_24BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_BGR888(x) GP_PIXEL_ADDR_OFFSET_24BPP(x)

/*
 * macros to create gp_pixel of pixel type BGR888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_BGR888(B, G, R) (0\
	+ ((B) << 16) \
	+ ((G) << 8) \
	+ ((R) << 0) \
	)

/* Automatically generated code for pixel type RGB555
 *
 * Size (bpp): 16 (16BPP)
 * Pixel structure: BBBBBGGGGGRRRRRx
 * Channels:
 *   R  offset:10 size:5
 *   G  offset:5 size:5
 *   B  offset:0 size:5
 */

/*
 * macros to get channels of pixel type RGB555
 */
#define GP_PIXEL_GET_R_RGB555(p) (GP_GET_BITS(10, 5, (p)))
#define GP_PIXEL_GET_G_RGB555(p) (GP_GET_BITS(5, 5, (p)))
#define GP_PIXEL_GET_B_RGB555(p) (GP_GET_BITS(0, 5, (p)))

/*
 * macros to get address and bit-offset of a pixel RGB555 in a pixmap
 */
#define GP_PIXEL_ADDR_RGB555(pixmap, x, y) GP_PIXEL_ADDR_16BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB555(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros to create gp_pixel of pixel type RGB555 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB555(R, G, B) (0\
	+ ((R) << 10) \
	+ ((G) << 5) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type RGB565
 *
 * Size (bpp): 16 (16BPP)
 * Pixel structure: BBBBBGGGGGGRRRRR
 * Channels:
 *   R  offset:11 size:5
 *   G  offset:5 size:6
 *   B  offset:0 size:5
 */

/*
 * macros to get channels of pixel type RGB565
 */
#define GP_PIXEL_GET_R_RGB565(p) (GP_GET_BITS(11, 5, (p)))
#define GP_PIXEL_GET_G_RGB565(p) (GP_GET_BITS(5, 6, (p)))
#define GP_PIXEL_GET_B_RGB565(p) (GP_GET_BITS(0, 5, (p)))

/*
 * macros to get address and bit-offset of a pixel RGB565 in a pixmap
 */
#define GP_PIXEL_ADDR_RGB565(pixmap, x, y) GP_PIXEL_ADDR_16BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB565(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros to create gp_pixel of pixel type RGB565 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB565(R, G, B) (0\
	+ ((R) << 11) \
	+ ((G) << 5) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type RGB666
 *
 * Size (bpp): 18 (18BPP_DB)
 * Pixel structure: BBBBBBGGGGGGRRRRRR
 * Bit order: DB
 * Channels:
 *   R  offset:12 size:6
 *   G  offset:6 size:6
 *   B  offset:0 size:6
 */

/*
 * macros to get channels of pixel type RGB666
 */
#define GP_PIXEL_GET_R_RGB666(p) (GP_GET_BITS(12, 6, (p)))
#define GP_PIXEL_GET_G_RGB666(p) (GP_GET_BITS(6, 6, (p)))
#define GP_PIXEL_GET_B_RGB666(p) (GP_GET_BITS(0, 6, (p)))

/*
 * macros to get address and bit-offset of a pixel RGB666 in a pixmap
 */
#define GP_PIXEL_ADDR_RGB666(pixmap, x, y) GP_PIXEL_ADDR_18BPP_DB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB666(x) GP_PIXEL_ADDR_OFFSET_18BPP_DB(x)

/*
 * macros to create gp_pixel of pixel type RGB666 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB666(R, G, B) (0\
	+ ((R) << 12) \
	+ ((G) << 6) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type RGB332
 *
 * Size (bpp): 8 (8BPP)
 * Pixel structure: BBGGGRRR
 * Channels:
 *   R  offset:5 size:3
 *   G  offset:2 size:3
 *   B  offset:0 size:2
 */

/*
 * macros to get channels of pixel type RGB332
 */
#define GP_PIXEL_GET_R_RGB332(p) (GP_GET_BITS(5, 3, (p)))
#define GP_PIXEL_GET_G_RGB332(p) (GP_GET_BITS(2, 3, (p)))
#define GP_PIXEL_GET_B_RGB332(p) (GP_GET_BITS(0, 2, (p)))

/*
 * macros to get address and bit-offset of a pixel RGB332 in a pixmap
 */
#define GP_PIXEL_ADDR_RGB332(pixmap, x, y) GP_PIXEL_ADDR_8BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB332(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros to create gp_pixel of pixel type RGB332 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB332(R, G, B) (0\
	+ ((R) << 5) \
	+ ((G) << 2) \
	+ ((B) << 0) \
	)

/* Automatically generated code for pixel type CMYK8888
 *
 * Size (bpp): 32 (32BPP)
 * Pixel structure: CCCCCCCCMMMMMMMMYYYYYYYYKKKKKKKK
 * Channels:
 *   K  offset:24 size:8
 *   Y  offset:16 size:8
 *   M  offset:8 size:8
 *   C  offset:0 size:8
 */

/*
 * macros to get channels of pixel type CMYK8888
 */
#define GP_PIXEL_GET_K_CMYK8888(p) (GP_GET_BITS(24, 8, (p)))
#define GP_PIXEL_GET_Y_CMYK8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_M_CMYK8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_C_CMYK8888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to get address and bit-offset of a pixel CMYK8888 in a pixmap
 */
#define GP_PIXEL_ADDR_CMYK8888(pixmap, x, y) GP_PIXEL_ADDR_32BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_CMYK8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros to create gp_pixel of pixel type CMYK8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_CMYK8888(K, Y, M, C) (0\
	+ ((K) << 24) \
	+ ((Y) << 16) \
	+ ((M) << 8) \
	+ ((C) << 0) \
	)

/* Automatically generated code for pixel type P2
 *
 * Size (bpp): 2 (2BPP_UB)
 * Pixel structure: PP
 * Bit order: UB
 * Channels:
 *   P  offset:0 size:2
 */

/*
 * macros to get channels of pixel type P2
 */
#define GP_PIXEL_GET_P_P2(p) (GP_GET_BITS(0, 2, (p)))

/*
 * macros to get address and bit-offset of a pixel P2 in a pixmap
 */
#define GP_PIXEL_ADDR_P2(pixmap, x, y) GP_PIXEL_ADDR_2BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_P2(x) GP_PIXEL_ADDR_OFFSET_2BPP_UB(x)

/*
 * macros to create gp_pixel of pixel type P2 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_P2(P) (0\
	+ ((P) << 0) \
	)

/* Automatically generated code for pixel type P4
 *
 * Size (bpp): 4 (4BPP_UB)
 * Pixel structure: PPPP
 * Bit order: UB
 * Channels:
 *   P  offset:0 size:4
 */

/*
 * macros to get channels of pixel type P4
 */
#define GP_PIXEL_GET_P_P4(p) (GP_GET_BITS(0, 4, (p)))

/*
 * macros to get address and bit-offset of a pixel P4 in a pixmap
 */
#define GP_PIXEL_ADDR_P4(pixmap, x, y) GP_PIXEL_ADDR_4BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_P4(x) GP_PIXEL_ADDR_OFFSET_4BPP_UB(x)

/*
 * macros to create gp_pixel of pixel type P4 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_P4(P) (0\
	+ ((P) << 0) \
	)

/* Automatically generated code for pixel type P8
 *
 * Size (bpp): 8 (8BPP)
 * Pixel structure: PPPPPPPP
 * Channels:
 *   P  offset:0 size:8
 */

/*
 * macros to get channels of pixel type P8
 */
#define GP_PIXEL_GET_P_P8(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to get address and bit-offset of a pixel P8 in a pixmap
 */
#define GP_PIXEL_ADDR_P8(pixmap, x, y) GP_PIXEL_ADDR_8BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_P8(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros to create gp_pixel of pixel type P8 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_P8(P) (0\
	+ ((P) << 0) \
	)

/* Automatically generated code for pixel type G1_DB
 *
 * Size (bpp): 1 (1BPP_DB)
 * Pixel structure: V
 * Bit order: DB
 * Channels:
 *   V  offset:0 size:1
 */

/*
 * macros to get channels of pixel type G1_DB
 */
#define GP_PIXEL_GET_V_G1_DB(p) (GP_GET_BITS(0, 1, (p)))

/*
 * macros to get address and bit-offset of a pixel G1_DB in a pixmap
 */
#define GP_PIXEL_ADDR_G1_DB(pixmap, x, y) GP_PIXEL_ADDR_1BPP_DB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G1_DB(x) GP_PIXEL_ADDR_OFFSET_1BPP_DB(x)

/*
 * macros to create gp_pixel of pixel type G1_DB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G1_DB(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G2_DB
 *
 * Size (bpp): 2 (2BPP_DB)
 * Pixel structure: VV
 * Bit order: DB
 * Channels:
 *   V  offset:0 size:2
 */

/*
 * macros to get channels of pixel type G2_DB
 */
#define GP_PIXEL_GET_V_G2_DB(p) (GP_GET_BITS(0, 2, (p)))

/*
 * macros to get address and bit-offset of a pixel G2_DB in a pixmap
 */
#define GP_PIXEL_ADDR_G2_DB(pixmap, x, y) GP_PIXEL_ADDR_2BPP_DB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G2_DB(x) GP_PIXEL_ADDR_OFFSET_2BPP_DB(x)

/*
 * macros to create gp_pixel of pixel type G2_DB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G2_DB(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G4_DB
 *
 * Size (bpp): 4 (4BPP_DB)
 * Pixel structure: VVVV
 * Bit order: DB
 * Channels:
 *   V  offset:0 size:4
 */

/*
 * macros to get channels of pixel type G4_DB
 */
#define GP_PIXEL_GET_V_G4_DB(p) (GP_GET_BITS(0, 4, (p)))

/*
 * macros to get address and bit-offset of a pixel G4_DB in a pixmap
 */
#define GP_PIXEL_ADDR_G4_DB(pixmap, x, y) GP_PIXEL_ADDR_4BPP_DB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G4_DB(x) GP_PIXEL_ADDR_OFFSET_4BPP_DB(x)

/*
 * macros to create gp_pixel of pixel type G4_DB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G4_DB(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G1_UB
 *
 * Size (bpp): 1 (1BPP_UB)
 * Pixel structure: V
 * Bit order: UB
 * Channels:
 *   V  offset:0 size:1
 */

/*
 * macros to get channels of pixel type G1_UB
 */
#define GP_PIXEL_GET_V_G1_UB(p) (GP_GET_BITS(0, 1, (p)))

/*
 * macros to get address and bit-offset of a pixel G1_UB in a pixmap
 */
#define GP_PIXEL_ADDR_G1_UB(pixmap, x, y) GP_PIXEL_ADDR_1BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G1_UB(x) GP_PIXEL_ADDR_OFFSET_1BPP_UB(x)

/*
 * macros to create gp_pixel of pixel type G1_UB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G1_UB(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G2_UB
 *
 * Size (bpp): 2 (2BPP_UB)
 * Pixel structure: VV
 * Bit order: UB
 * Channels:
 *   V  offset:0 size:2
 */

/*
 * macros to get channels of pixel type G2_UB
 */
#define GP_PIXEL_GET_V_G2_UB(p) (GP_GET_BITS(0, 2, (p)))

/*
 * macros to get address and bit-offset of a pixel G2_UB in a pixmap
 */
#define GP_PIXEL_ADDR_G2_UB(pixmap, x, y) GP_PIXEL_ADDR_2BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G2_UB(x) GP_PIXEL_ADDR_OFFSET_2BPP_UB(x)

/*
 * macros to create gp_pixel of pixel type G2_UB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G2_UB(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G4_UB
 *
 * Size (bpp): 4 (4BPP_UB)
 * Pixel structure: VVVV
 * Bit order: UB
 * Channels:
 *   V  offset:0 size:4
 */

/*
 * macros to get channels of pixel type G4_UB
 */
#define GP_PIXEL_GET_V_G4_UB(p) (GP_GET_BITS(0, 4, (p)))

/*
 * macros to get address and bit-offset of a pixel G4_UB in a pixmap
 */
#define GP_PIXEL_ADDR_G4_UB(pixmap, x, y) GP_PIXEL_ADDR_4BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G4_UB(x) GP_PIXEL_ADDR_OFFSET_4BPP_UB(x)

/*
 * macros to create gp_pixel of pixel type G4_UB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G4_UB(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type G8
 *
 * Size (bpp): 8 (8BPP)
 * Pixel structure: VVVVVVVV
 * Channels:
 *   V  offset:0 size:8
 */

/*
 * macros to get channels of pixel type G8
 */
#define GP_PIXEL_GET_V_G8(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to get address and bit-offset of a pixel G8 in a pixmap
 */
#define GP_PIXEL_ADDR_G8(pixmap, x, y) GP_PIXEL_ADDR_8BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G8(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros to create gp_pixel of pixel type G8 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G8(V) (0\
	+ ((V) << 0) \
	)

/* Automatically generated code for pixel type GA88
 *
 * Size (bpp): 16 (16BPP)
 * Pixel structure: VVVVVVVVAAAAAAAA
 * Channels:
 *   V  offset:0 size:8
 *   A  offset:8 size:8
 */

/*
 * macros to get channels of pixel type GA88
 */
#define GP_PIXEL_GET_V_GA88(p) (GP_GET_BITS(0, 8, (p)))
#define GP_PIXEL_GET_A_GA88(p) (GP_GET_BITS(8, 8, (p)))

/*
 * macros to get address and bit-offset of a pixel GA88 in a pixmap
 */
#define GP_PIXEL_ADDR_GA88(pixmap, x, y) GP_PIXEL_ADDR_16BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_GA88(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros to create gp_pixel of pixel type GA88 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_GA88(V, A) (0\
	+ ((V) << 0) \
	+ ((A) << 8) \
	)

/* Automatically generated code for pixel type G16
 *
 * Size (bpp): 16 (16BPP)
 * Pixel structure: VVVVVVVVVVVVVVVV
 * Channels:
 *   V  offset:0 size:16
 */

/*
 * macros to get channels of pixel type G16
 */
#define GP_PIXEL_GET_V_G16(p) (GP_GET_BITS(0, 16, (p)))

/*
 * macros to get address and bit-offset of a pixel G16 in a pixmap
 */
#define GP_PIXEL_ADDR_G16(pixmap, x, y) GP_PIXEL_ADDR_16BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G16(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros to create gp_pixel of pixel type G16 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G16(V) (0\
	+ ((V) << 0) \
	)


/*
 * macros for branching on pixel_type
 */

#define GP_FN_PER_PIXELTYPE(FN_NAME, type, ...)\
	switch (type) { \
		case GP_PIXEL_xRGB8888:\
			FN_NAME##_xRGB8888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGBA8888:\
			FN_NAME##_RGBA8888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB888:\
			FN_NAME##_RGB888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_BGR888:\
			FN_NAME##_BGR888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB555:\
			FN_NAME##_RGB555(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB565:\
			FN_NAME##_RGB565(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB666:\
			FN_NAME##_RGB666(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB332:\
			FN_NAME##_RGB332(__VA_ARGS__);\
		break;\
		case GP_PIXEL_CMYK8888:\
			FN_NAME##_CMYK8888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_P2:\
			FN_NAME##_P2(__VA_ARGS__);\
		break;\
		case GP_PIXEL_P4:\
			FN_NAME##_P4(__VA_ARGS__);\
		break;\
		case GP_PIXEL_P8:\
			FN_NAME##_P8(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G1_DB:\
			FN_NAME##_G1_DB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G2_DB:\
			FN_NAME##_G2_DB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G4_DB:\
			FN_NAME##_G4_DB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G1_UB:\
			FN_NAME##_G1_UB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G2_UB:\
			FN_NAME##_G2_UB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G4_UB:\
			FN_NAME##_G4_UB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G8:\
			FN_NAME##_G8(__VA_ARGS__);\
		break;\
		case GP_PIXEL_GA88:\
			FN_NAME##_GA88(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G16:\
			FN_NAME##_G16(__VA_ARGS__);\
		break;\
		default: GP_ABORT("Invalid PixelType %d", type);\
	}

#define GP_FN_RET_PER_PIXELTYPE(FN_NAME, type, ...)\
	switch (type) { \
		case GP_PIXEL_xRGB8888:\
			return FN_NAME##_xRGB8888(__VA_ARGS__);\
		case GP_PIXEL_RGBA8888:\
			return FN_NAME##_RGBA8888(__VA_ARGS__);\
		case GP_PIXEL_RGB888:\
			return FN_NAME##_RGB888(__VA_ARGS__);\
		case GP_PIXEL_BGR888:\
			return FN_NAME##_BGR888(__VA_ARGS__);\
		case GP_PIXEL_RGB555:\
			return FN_NAME##_RGB555(__VA_ARGS__);\
		case GP_PIXEL_RGB565:\
			return FN_NAME##_RGB565(__VA_ARGS__);\
		case GP_PIXEL_RGB666:\
			return FN_NAME##_RGB666(__VA_ARGS__);\
		case GP_PIXEL_RGB332:\
			return FN_NAME##_RGB332(__VA_ARGS__);\
		case GP_PIXEL_CMYK8888:\
			return FN_NAME##_CMYK8888(__VA_ARGS__);\
		case GP_PIXEL_P2:\
			return FN_NAME##_P2(__VA_ARGS__);\
		case GP_PIXEL_P4:\
			return FN_NAME##_P4(__VA_ARGS__);\
		case GP_PIXEL_P8:\
			return FN_NAME##_P8(__VA_ARGS__);\
		case GP_PIXEL_G1_DB:\
			return FN_NAME##_G1_DB(__VA_ARGS__);\
		case GP_PIXEL_G2_DB:\
			return FN_NAME##_G2_DB(__VA_ARGS__);\
		case GP_PIXEL_G4_DB:\
			return FN_NAME##_G4_DB(__VA_ARGS__);\
		case GP_PIXEL_G1_UB:\
			return FN_NAME##_G1_UB(__VA_ARGS__);\
		case GP_PIXEL_G2_UB:\
			return FN_NAME##_G2_UB(__VA_ARGS__);\
		case GP_PIXEL_G4_UB:\
			return FN_NAME##_G4_UB(__VA_ARGS__);\
		case GP_PIXEL_G8:\
			return FN_NAME##_G8(__VA_ARGS__);\
		case GP_PIXEL_GA88:\
			return FN_NAME##_GA88(__VA_ARGS__);\
		case GP_PIXEL_G16:\
			return FN_NAME##_G16(__VA_ARGS__);\
		default: GP_ABORT("Invalid PixelType %d", type);\
	}


/*
 * macros to do per-channel operations on a pixel.
 */
#define GP_PIXEL_CHANS_ADD_xRGB8888(pixel, perc) \
	GP_PIXEL_CREATE_xRGB8888( \
		GP_SAT_ADD(GP_PIXEL_GET_R_xRGB8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_xRGB8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_B_xRGB8888(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_RGBA8888(pixel, perc) \
	GP_PIXEL_CREATE_RGBA8888( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGBA8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGBA8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGBA8888(pixel), 0xff * perc / 100, 0xff), \
		GP_PIXEL_GET_A_RGBA8888(pixel) \
	)

#define GP_PIXEL_CHANS_ADD_RGB888(pixel, perc) \
	GP_PIXEL_CREATE_RGB888( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB888(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_BGR888(pixel, perc) \
	GP_PIXEL_CREATE_BGR888( \
		GP_SAT_ADD(GP_PIXEL_GET_B_BGR888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_BGR888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_R_BGR888(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_RGB555(pixel, perc) \
	GP_PIXEL_CREATE_RGB555( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB555(pixel), 0x1f * perc / 100, 0x1f), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB555(pixel), 0x1f * perc / 100, 0x1f), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB555(pixel), 0x1f * perc / 100, 0x1f) \
	)

#define GP_PIXEL_CHANS_ADD_RGB565(pixel, perc) \
	GP_PIXEL_CREATE_RGB565( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB565(pixel), 0x1f * perc / 100, 0x1f), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB565(pixel), 0x3f * perc / 100, 0x3f), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB565(pixel), 0x1f * perc / 100, 0x1f) \
	)

#define GP_PIXEL_CHANS_ADD_RGB666(pixel, perc) \
	GP_PIXEL_CREATE_RGB666( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB666(pixel), 0x3f * perc / 100, 0x3f), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB666(pixel), 0x3f * perc / 100, 0x3f), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB666(pixel), 0x3f * perc / 100, 0x3f) \
	)

#define GP_PIXEL_CHANS_ADD_RGB332(pixel, perc) \
	GP_PIXEL_CREATE_RGB332( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB332(pixel), 0x7 * perc / 100, 0x7), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB332(pixel), 0x7 * perc / 100, 0x7), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB332(pixel), 0x3 * perc / 100, 0x3) \
	)

#define GP_PIXEL_CHANS_ADD_CMYK8888(pixel, perc) \
	GP_PIXEL_CREATE_CMYK8888( \
		GP_SAT_ADD(GP_PIXEL_GET_K_CMYK8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_Y_CMYK8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_M_CMYK8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_C_CMYK8888(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_P2(pixel, perc) \
	GP_PIXEL_CREATE_P2( \
		GP_SAT_ADD(GP_PIXEL_GET_P_P2(pixel), 0x3 * perc / 100, 0x3) \
	)

#define GP_PIXEL_CHANS_ADD_P4(pixel, perc) \
	GP_PIXEL_CREATE_P4( \
		GP_SAT_ADD(GP_PIXEL_GET_P_P4(pixel), 0xf * perc / 100, 0xf) \
	)

#define GP_PIXEL_CHANS_ADD_P8(pixel, perc) \
	GP_PIXEL_CREATE_P8( \
		GP_SAT_ADD(GP_PIXEL_GET_P_P8(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_G1_DB(pixel, perc) \
	GP_PIXEL_CREATE_G1_DB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G1_DB(pixel), 0x1 * perc / 100, 0x1) \
	)

#define GP_PIXEL_CHANS_ADD_G2_DB(pixel, perc) \
	GP_PIXEL_CREATE_G2_DB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G2_DB(pixel), 0x3 * perc / 100, 0x3) \
	)

#define GP_PIXEL_CHANS_ADD_G4_DB(pixel, perc) \
	GP_PIXEL_CREATE_G4_DB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G4_DB(pixel), 0xf * perc / 100, 0xf) \
	)

#define GP_PIXEL_CHANS_ADD_G1_UB(pixel, perc) \
	GP_PIXEL_CREATE_G1_UB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G1_UB(pixel), 0x1 * perc / 100, 0x1) \
	)

#define GP_PIXEL_CHANS_ADD_G2_UB(pixel, perc) \
	GP_PIXEL_CREATE_G2_UB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G2_UB(pixel), 0x3 * perc / 100, 0x3) \
	)

#define GP_PIXEL_CHANS_ADD_G4_UB(pixel, perc) \
	GP_PIXEL_CREATE_G4_UB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G4_UB(pixel), 0xf * perc / 100, 0xf) \
	)

#define GP_PIXEL_CHANS_ADD_G8(pixel, perc) \
	GP_PIXEL_CREATE_G8( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G8(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_GA88(pixel, perc) \
	GP_PIXEL_CREATE_GA88( \
		GP_SAT_ADD(GP_PIXEL_GET_V_GA88(pixel), 0xff * perc / 100, 0xff), \
		GP_PIXEL_GET_A_GA88(pixel) \
	)

#define GP_PIXEL_CHANS_ADD_G16(pixel, perc) \
	GP_PIXEL_CREATE_G16( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G16(pixel), 0xffff * perc / 100, 0xffff) \
	)

#endif /* GP_PIXEL_GEN_H */
