// SPDX-License-Identifier: GPL-2.1-or-later
/*
 * gp_pixel_pack.gen.h
 *
 * GENERATED on 2023 10 30 20:24:52 from gp_pixel_pack.gen.h.t
 *
 * DO NOT MODIFY THIS FILE DIRECTLY!
 */
#ifndef GP_PIXEL_PACK_GEN_H
#define GP_PIXEL_PACK_GEN_H

/*
 * Copyright (C) 2023 Cyril Hrubis <metan@ucw.cz>
 */

/*
 * Pixel packing defines how pixels are packed in a stream of bytes. Note that
 * there may be more than one packing defined for a single bit depth.
 *
 * The bits_per_pixel (pixel size) defines how big is a pixel, it's useful for
 * determining the buffer size, but it does not tell us anything about how
 * pixels are stacked in the buffer. Note that for 4bit depth there are two
 * ways how to interpret which half byte goes first. So there will always be
 * more different packings than pixel sizes.
 */
typedef enum gp_pixel_pack {
	GP_PIXEL_PACK_1BPP_DB = 0x71,
	GP_PIXEL_PACK_1BPP_UB = 0x01,
	GP_PIXEL_PACK_2BPP_DB = 0x72,
	GP_PIXEL_PACK_2BPP_UB = 0x02,
	GP_PIXEL_PACK_4BPP_DB = 0x74,
	GP_PIXEL_PACK_4BPP_UB = 0x04,
	GP_PIXEL_PACK_8BPP = 0x08,
	GP_PIXEL_PACK_16BPP = 0x10,
	GP_PIXEL_PACK_24BPP = 0x18,
	GP_PIXEL_PACK_32BPP = 0x20,
	GP_PIXEL_PACK_18BPP_DB = 0x82,
} gp_pixel_pack;

#define GP_PIXEL_PACK_1BPP_DB GP_PIXEL_PACK_1BPP_DB
#define GP_PIXEL_PACK_1BPP_UB GP_PIXEL_PACK_1BPP_UB
#define GP_PIXEL_PACK_2BPP_DB GP_PIXEL_PACK_2BPP_DB
#define GP_PIXEL_PACK_2BPP_UB GP_PIXEL_PACK_2BPP_UB
#define GP_PIXEL_PACK_4BPP_DB GP_PIXEL_PACK_4BPP_DB
#define GP_PIXEL_PACK_4BPP_UB GP_PIXEL_PACK_4BPP_UB
#define GP_PIXEL_PACK_8BPP GP_PIXEL_PACK_8BPP
#define GP_PIXEL_PACK_16BPP GP_PIXEL_PACK_16BPP
#define GP_PIXEL_PACK_24BPP GP_PIXEL_PACK_24BPP
#define GP_PIXEL_PACK_32BPP GP_PIXEL_PACK_32BPP
#define GP_PIXEL_PACK_18BPP_DB GP_PIXEL_PACK_18BPP_DB

/*
 * Returns pixel size, e.g. bits per pixel given pixel packing.
 */
uint8_t gp_pixel_bpp_by_pack(enum gp_pixel_pack pack);

/*
 * Branch on pixel packing.
 */
#define GP_FN_RET_PER_PACK(FN_NAME, pack, ...) \
	switch (pack) { \
	case GP_PIXEL_PACK_1BPP_DB: \
		return FN_NAME##_1BPP_DB(__VA_ARGS__); \
	case GP_PIXEL_PACK_1BPP_UB: \
		return FN_NAME##_1BPP_UB(__VA_ARGS__); \
	case GP_PIXEL_PACK_2BPP_DB: \
		return FN_NAME##_2BPP_DB(__VA_ARGS__); \
	case GP_PIXEL_PACK_2BPP_UB: \
		return FN_NAME##_2BPP_UB(__VA_ARGS__); \
	case GP_PIXEL_PACK_4BPP_DB: \
		return FN_NAME##_4BPP_DB(__VA_ARGS__); \
	case GP_PIXEL_PACK_4BPP_UB: \
		return FN_NAME##_4BPP_UB(__VA_ARGS__); \
	case GP_PIXEL_PACK_8BPP: \
		return FN_NAME##_8BPP(__VA_ARGS__); \
	case GP_PIXEL_PACK_16BPP: \
		return FN_NAME##_16BPP(__VA_ARGS__); \
	case GP_PIXEL_PACK_24BPP: \
		return FN_NAME##_24BPP(__VA_ARGS__); \
	case GP_PIXEL_PACK_32BPP: \
		return FN_NAME##_32BPP(__VA_ARGS__); \
	case GP_PIXEL_PACK_18BPP_DB: \
		return FN_NAME##_18BPP_DB(__VA_ARGS__); \
	}

/*
 * Branch on pixel packing.
 */
#define GP_FN_PER_PACK(FN_NAME, pack, ...) \
	switch (pack) { \
	case GP_PIXEL_PACK_1BPP_DB: \
		FN_NAME##_1BPP_DB(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_1BPP_UB: \
		FN_NAME##_1BPP_UB(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_2BPP_DB: \
		FN_NAME##_2BPP_DB(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_2BPP_UB: \
		FN_NAME##_2BPP_UB(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_4BPP_DB: \
		FN_NAME##_4BPP_DB(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_4BPP_UB: \
		FN_NAME##_4BPP_UB(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_8BPP: \
		FN_NAME##_8BPP(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_16BPP: \
		FN_NAME##_16BPP(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_24BPP: \
		FN_NAME##_24BPP(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_32BPP: \
		FN_NAME##_32BPP(__VA_ARGS__); \
	break; \
	case GP_PIXEL_PACK_18BPP_DB: \
		FN_NAME##_18BPP_DB(__VA_ARGS__); \
	break; \
	}

/*
 * Branch on gp_pixel_type argument.
 */
#define GP_FN_PER_PACK_PIXELTYPE(FN_NAME, type, ...) \
	GP_FN_PER_PACK(FN_NAME, gp_pixel_types[type].pack, __VA_ARGS__)

/*
 * Branch on gp_pixel_type argument.
 */
#define GP_FN_RET_PER_PACK_PIXELTYPE(FN_NAME, type, ...) \
	GP_FN_RET_PER_PACK(FN_NAME, gp_pixel_types[type].pack, __VA_ARGS__)

/*
 * Branch on pixmap argument.
 */
#define GP_FN_PER_PACK_PIXMAP(FN_NAME, pixmap, ...) \
	GP_FN_PER_PACK(FN_NAME, gp_pixel_types[pixmap->pixel_type].pack, __VA_ARGS__)

/*
 * Branch on pixmap argument.
 */
#define GP_FN_RET_PER_PACK_PIXMAP(FN_NAME, pixmap, ...) \
	GP_FN_RET_PER_PACK(FN_NAME, gp_pixel_types[pixmap->pixel_type].pack, __VA_ARGS__)

//TODO REMOVE WHEN LAST ALGO IS CONVERTED
#define GP_DEF_DRAW_FN_PER_PACK(fname, MACRO_NAME) \
	GP_DEF_FN_PER_PACK(fname, MACRO_NAME, gp_putpixel_raw_clipped_)

#define GP_DEF_FN_PER_PACK(fname, MACRO_NAME, fdraw) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 1BPP_DB) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 1BPP_UB) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 2BPP_DB) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 2BPP_UB) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 4BPP_DB) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 4BPP_UB) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 8BPP) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 16BPP) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 24BPP) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 32BPP) \
        GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, 18BPP_DB) \

#define GP_DEF_FN_FOR_PACK(fname, MACRO_NAME, fdraw, bpp) \
        MACRO_NAME(fname##_##bpp, gp_pixmap *, gp_pixel, fdraw##bpp)

#define GP_DEF_FILL_FN_PER_PACK(fname, MACRO_NAME) \
        GP_DEF_FN_PER_PACK(fname, MACRO_NAME, gp_hline_raw_)

#endif /* GP_PIXEL_PACK_GEN_H */
